import {
  HttpStatus,
  Inject,
  Injectable,
  Logger,
  UnauthorizedException,
} from '@nestjs/common';
import { validatePassword } from 'src/core/helpers/password/bcrypt.helpers';
import { BaseCustomManager } from 'src/core/modules/domain/usecase/managers/base-custom.manager';
import { SessionService } from 'src/core/sessions';
import { STATUS } from 'src/core/strings/constants/base.constants';
import { EventTopics } from 'src/core/strings/constants/interface.constants';
import { UserModel } from 'src/modules/user-related/user/data/models/user.model';
import { UserEntity } from 'src/modules/user-related/user/domain/entities/user.entity';
import { In } from 'typeorm';
import { UserRole } from 'src/modules/user-related/user/constants';
import { LogUserType } from 'src/core/helpers/constant';
import { LogUserLoginEvent } from 'src/modules/configuration/log/domain/entities/log-user-login.event';

@Injectable()
export class LoginAdminQueueManager extends BaseCustomManager<UserEntity> {
  @Inject()
  protected session: SessionService;
  protected token;
  protected userLogin;

  async validateProcess(): Promise<void> {
    return;
  }

  async beforeProcess(): Promise<void> {
    return;
  }

  async process(): Promise<void> {
    // get user active by username
    this.userLogin = await this.dataService.getOneByOptions({
      where: {
        username: this.data.username,
        status: STATUS.ACTIVE,
        role: In([UserRole.QUEUE_ADMIN, UserRole.SUPERADMIN]),
      },
      relations: ['user_login'],
    });
    if (!this.userLogin) this.throwError();

    // validasi password
    const valid = await validatePassword(
      this.data.password,
      this.userLogin?.password,
    );
    if (!valid) this.throwError();

    if (
      this.userLogin.user_login &&
      this.userLogin.role === UserRole.QUEUE_ADMIN
    ) {
      throw new UnauthorizedException({
        statusCode: HttpStatus.UNAUTHORIZED,
        message: `Gagal! akun anda sudah login di perangkat lain.`,
        error: 'Unauthorized',
      });
    }

    // * Disini untuk isi token
    const tokenData = {
      id: this.userLogin.id,
      name: this.userLogin.name,
      username: this.userLogin.username,
      role: this.userLogin.role,
      user_privilege_id: this.userLogin.user_privilege_id,
    };

    Logger.debug('Sign Token Admin Queue', 'LoginAdminQueueManager');
    this.token = this.session.createAccessToken(tokenData);

    Logger.debug('refreshToken Admin Queue', 'LoginAdminQueueManager');
    const refreshToken = this.session.createAccessToken(tokenData);

    Logger.debug('Update Refresh Token Admin Queue', 'LoginAdminQueueManager');

    const newDataUser = { refresh_token: refreshToken };
    if (this.userLogin.role === UserRole.QUEUE_ADMIN) {
      Object.assign(newDataUser, {
        user_login: {
          user_id: this.userLogin.id,
          login_token: this.token,
          login_date: new Date().getTime(),
        },
      });
    }
    // Update refresh token
    await this.dataService.update(
      this.queryRunner,
      this.entityTarget,
      { id: this.userLogin.id },
      newDataUser,
    );
    await this.publishEvents();

    Logger.debug('Process Login Admin Queue Done', 'LoginAdminQueueManager');
    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  getResult() {
    return {
      id: this.userLogin.id,
      name: this.userLogin.name,
      username: this.userLogin.username,
      role: this.userLogin.role,
      token: this.token,
    };
  }

  get entityTarget(): any {
    return UserModel;
  }

  get eventTopics(): EventTopics[] {
    return [
      {
        topic: LogUserLoginEvent,
        data: {
          type: LogUserType.login,
          role: this.userLogin.role,
          user_id: this.userLogin.id,
          username: this.userLogin.username,
          created_at: new Date().getTime(),
        },
      },
    ];
  }

  // !throw errornya akan sama, untuk security
  throwError() {
    throw new UnauthorizedException({
      statusCode: HttpStatus.UNAUTHORIZED,
      message: `Gagal! username atau password tidak sesuai`,
      error: 'Unauthorized',
    });
  }
}
